@@ -642,57 +642,64 @@ sub _integrate_pending_revs {
    my @to_resolve;
    my $max_time;
    for my $r ( @{$self->{P4_PENDING}} ) {
-      next unless $r->is_branch_rev || $r->is_clone_rev;
 
-      my $from_id = $r->from_id;
-      $from_id = $r->previous_id if empty $from_id;
+      my $pr_id = $r->previous_id;
 
-      die "No previous_id or from_id in ", $r->action,
-         " rev: ", $r->as_string, "\n"
-         if empty $from_id;
+      if ( ! empty $pr_id ) {
+         my $fn = $r->name ;
+         my $work_path = $self->work_path( "co", $fn ) ;
+         my $denorm_name = $self->denormalize_name( $fn );
 
-      my $fn = $r->name ;
-      my $work_path = $self->work_path( "co", $fn ) ;
-      my $denorm_name = $self->denormalize_name( $fn );
+         my ( $pfull_name, $prev_id, $pstate, $pbranch_id ) =
+            $self->rev_map->get( [ $r->source_repo_id, $pr_id ] );
 
-      my ( $pfull_name, $prev_id, $pstate ) =
-         $self->rev_map->get( [ $r->source_repo_id, $from_id ] );
+         die "Can't integrate from unknown revision:\n",
+            "parent: ", $pr_id, "\n",
+            "child:  ", $r->as_string, "\n"
+            unless defined $pstate;
 
-      die "Can't integrate from unknown revision:\n",
-         "parent: ", $from_id, "\n",
-         "child:  ", $r->as_string, "\n"
-         unless defined $pstate;
+         next if !$r->is_branch_rev && $pbranch_id eq ( $r->branch_id || "" );
+             ## We check both is_branch_rev to catch branches between
+             ## files in the same branch.  We check branch_ids to catch
+             ## branches that did not start with a "branch" action.
+             ## This latter case arises now because RevML does not yet
+             ## have a "<branch/>" replacement for <placeholder/>, but
+             ## it could be argued that this logic could give VCP the
+             ## ability to use Map or StringEdit filters to branch files
+             ## somehow.  That's a stretch and perhaps the "eq" test
+             ## should be removed when RevML gets a <branch/> action.
 
-      ## Branch the previous version to make the new one.  Leave
-      ## add_it set so we can drop the new one in over the
-      ## branched version in case it's changed.
-      die "branched revision has same name as parent ('$pfull_name'):\n",
-          "    parent:   ", $from_id, " ", $pstate, " ", "\n",
-          "    branched: ", $r->as_string, "\n",
-          "Perhaps a Map: filter is missing/broken\n"
-         if $denorm_name eq $pfull_name;
+         ## Branch the previous version to make the new one.  Leave
+         ## add_it set so we can drop the new one in over the
+         ## branched version in case it's changed.
+         die "branched revision has same name as parent ('$pfull_name'):\n",
+             "    parent:   ", $pr_id, "\n",
+             "    branched: ", $r->as_string, "\n",
+             "Perhaps a Map: filter is missing/broken\n"
+            if $denorm_name eq $pfull_name;
+
+         if ( $pstate eq "deleted" && $r->is_branch_rev ) {
+            ## CVS allows branching from "deleted" revisions,
+            ## let's hope that the revision before the previous
+            ## revision exists and is *not* deleted.  We only
+            ## do this on branch creation, not when integrating
+            ## to clones; in that case we want to clone the deletion
+            ## operation.
+            $prev_id--;
+         }
+
+         push @to_integrate, [
+            $r,
+            $fn,
+            $denorm_name,
+            $work_path,
+            $pfull_name,
+            $prev_id,
+            $pstate,
+         ];
+         $max_time = $r->time if ! defined $max_time || $r->time > $max_time ;
 
-      if ( $pstate eq "deleted" && $r->is_branch_rev ) {
-         ## CVS allows branching from "deleted" revisions,
-         ## let's hope that the revision before the previous
-         ## revision exists and is *not* deleted.  We only
-         ## do this on branch creation, not when integrating
-         ## to clones; in that case we want to clone the deletion
-         ## operation.
-         $prev_id--;
       }
-
-      push @to_integrate, [
-         $r,
-         $fn,
-         $denorm_name,
-         $work_path,
-         $pfull_name,
-         $prev_id,
-         $pstate,
-      ];
-
-      $max_time = $r->time if ! defined $max_time || $r->time > $max_time ;
    }
 
    return unless @to_integrate;
@@ -751,16 +758,17 @@ sub _integrate_pending_revs {
       my ( $r, $fn, $denorm_name, $work_path, $pfull_name, $prev_id, $pstate )
          = @$i;
 
-      my $is_delete = $r->is_clone_rev && $pstate eq "deleted";
+      my $is_delete = (
+         $r->action eq "delete"
+         || ( $r->is_clone_rev && $pstate eq "deleted" )
+      );
 
       push @to_resolve, $fn
-         if $self->files->exists( [ $denorm_name ] )
-            && !$is_delete
-            && ($self->files->get( [ $denorm_name ] ))[0] ne "deleted";
+         if $self->files->exists( [ $denorm_name ] ) && ! $is_delete;
 
-      $self->files->set( [ $denorm_name ], $pstate );
+      $self->files->set( [ $denorm_name ], "integrated to" );
 
-      push @revs_to_submit, $r;
+      push @revs_to_submit, $r unless $is_delete;
    }
 
    $self->p4( [ "resolve", "-at" ] ) if @to_resolve;
