@@ -16,8 +16,6 @@ VCP::Dest::revml - Outputs versioned files to a revml file
 
 =head1 OPTIONS
 
-=over
-
 =item --dtd
 
    --dtd=revml.dtd
@@ -328,8 +326,7 @@ sub handle_rev {
       $w->name(                 $fn                      );
       $w->source_name(          $r->source_name          );
       $w->source_filebranch_id( $r->source_filebranch_id );
-      $w->source_repo_id(       $r->source_repo_id       );
-      $w->action(               $r->action               );
+      $w->source_repo_id(       $r->source_repo_id );
       $w->type(       $r->type               ) if defined $r->type ;
       $w->p4_info(    $r->p4_info            ) if defined $r->p4_info ;
       $w->cvs_info(   $r->cvs_info           ) if defined $r->cvs_info ;
@@ -371,10 +368,6 @@ sub handle_rev {
       }
 
       $w->previous_id( $r->previous_id ) if defined $r->previous_id;
-      $w->from_id(     $r->from_id     ) if defined $r->from_id;
-      ## Sorted for readability & testability
-      $w->earlier_id( $_ ) for sort $r->earlier_ids ;
-
 
       my $convert_crs = $^O =~ /Win32/ && ( $r->type || "" ) eq "text" ;
 
@@ -383,7 +376,7 @@ sub handle_rev {
       my $cp;
 
       $cp = $r->get_source_file
-         if $r->is_real_rev;
+         if $is_base_rev || $r->action eq "add" || $r->action eq "edit";
 
       $self->{FILES}->{$r->id} = $cp unless empty $cp;
 
@@ -394,9 +387,11 @@ sub handle_rev {
          $close_it = 1 ;
       }
       elsif ( $r->is_placeholder_rev ) {
+         $w->placeholder() ;
          $digestion = 0;
       }
       elsif ( $r->action eq 'delete' ) {
+         $w->delete() ;
       }
       else {
          BUG "\$r->get_source_file() returned undef" unless defined $cp;
