@@ -205,12 +205,6 @@ condition.  This index maps unique keys to changes.
 This is an index of all revisions that are direct offspring of a
 revision.
 
-=head3 the PREDECESSOR_COUNT statistic
-
-Counts the number of parents a revision has that haven't been
-submitted yet.  A revision may have a previous_id and, optionally,
-also have a from_id (can't have a from_id without a previous_id,
-however).
 
 =head3 The REVS_BY_CHANGE_ID index
 
@@ -601,27 +595,9 @@ sub handle_rev {
    }
    else {
       ## It's a descendant node, note its parentage and stow it for later
-      $self->{CHILDREN}->[
-         $self->r_index( $r->previous_id )
-      ] .= $r_index_in_binary;
-
-      if ( !empty $r->from_id ) {
-         $self->{CHILDREN}->[
-            $self->r_index( $r->from_id )
-         ] .= $r_index_in_binary;
-         $self->{PREDECESSOR_COUNT}->[$r_index] = 2;
-      }
-      else {
-         $self->{PREDECESSOR_COUNT}->[$r_index] = 1;
-      }
-
-      for my $id ( $r->earlier_ids ) {
-         $self->{CHILDREN}->[
-            $self->r_index( $id )
-         ] .= $r_index_in_binary;
-         $self->{PREDECESSOR_COUNT}->[$r_index]++;
-      }
-
+        $self->{CHILDREN}->[
+           $self->r_index( $r->previous_id )
+        ] .= $r_index_in_binary;
    }
 
    ++$self->{REV_COUNT};
@@ -806,15 +782,9 @@ sub split_and_send_changes {
          my $r_index = delete $self->{INDEXES_BY_ID}->{$r->id};
 
          if ( exists $children->[$r_index] ) {
-            my @children = unpack "w*", $children->[$r_index];
-            for my $child ( @children ) {
-               --$self->{PREDECESSOR_COUNT}->[$child]
-                  if $self->{PREDECESSOR_COUNT}->[$child];
-               push @cur_indexes, $child
-                  unless $self->{PREDECESSOR_COUNT}->[$child];
-            }
-            undef $children->[$r_index];  ## undef $foo releases extra memory
+            push @cur_indexes, unpack "w*", $children->[$r_index];
          }
+         undef $children->[$r_index];  ## undef $foo releases extra memory
          undef $self->{CHANGES_BY_REV}->[$r_index];
       }
 
