@@ -55,16 +55,16 @@ BEGIN {
    ##    @=it's an array (needed for serialization support)
    ##
    'ID:_',                 ## A unique identifier for the rev
-   'NAME',                 ## The file name, relative to REV_ROOT
+   'NAME',               ## The file name, relative to REV_ROOT
    'SOURCE_NAME',          ## immutable field, initialized to NAME
    'SOURCE_FILEBRANCH_ID', ## immutable field, initialized to
                            ## NAME or NAME<branch_number> for cvs
    'SOURCE_REPO_ID',       ## immutable field, initialized to
                            ## <repo_type>:<repo_server>
    'TYPE',                 ## Type.  Binary/text.
-   'BRANCH_ID',            ## What branch this revision is on
+   'BRANCH_ID',          ## What branch this revision is on
    'SOURCE_BRANCH_ID',     ## immutable field initialized to BRANCH_ID
-   'REV_ID',               ## The source rep's ID for this revision
+   'REV_ID',             ## The source repositories unique ID for this revision
    'SOURCE_REV_ID',        ## immutable field initialized to REV_ID
    'CHANGE_ID',            ## The unique ID for the change set, if any
    'SOURCE_CHANGE_ID',     ## immutable field initialized to CHANGE_ID
@@ -72,20 +72,17 @@ BEGIN {
    'CVS_INFO',             ## cvs-specific info.
    'TIME:i',               ## The commit/submit time, in seconds-since-the-epoch
    'MOD_TIME:i',           ## The last modification time, if available
-   'USER_ID',              ## The submitter/commiter of the revision
-   'LABELS:_@',            ## A list of tags/labels assoc. with this rev.
-   'COMMENT',              ## The comment/message for this rev.
+   'USER_ID',            ## The submitter/commiter of the revision
+   'LABELS:_@',            ## A bit vector of tags/labels assoc. with this rev.
+   'COMMENT',            ## The comment/message for this rev.
    'ACTION',               ## What was done ('edit', 'move', 'delete', etc.)
-   'PREVIOUS_ID',          ## The id of the preceding version on this filebranch
-   'FROM_ID',              ## The id of a revision to clone or merge
-                           ## with the previous_id
-   'EARLIER_IDS:_@',       ## IDs for revs that occured before this one
-   'SOURCE:_',             ## A reference to the source so that the destination
-                           ## can get the file it needs right from the source.
-                           ## NOTE: it's up to callers to thunk this for
-                           ## serialization, not VCP::Rev.  Some will want
-                           ## to undef() it, others will want to save
-                           ## and restore it.
+   'PREVIOUS_ID',          ## The id of the preceding version
+   'SOURCE:_',            ## A reference to the source so that the destination
+                          ## can get the file it needs right from the source.
+                          ## NOTE: it's up to callers to thunk this for
+                          ## serialization, not VCP::Rev.  Some will want
+                          ## to undef() it, others will want to save
+                          ## and restore it.
 );
 
 ##
@@ -167,14 +164,6 @@ sub new {
       $self->{seen_labels} = {};
    }
 
-   if ( $self->{earlier_ids} ) {
-      $self->set_earlier_ids( @{delete $self->{earlier_ids}} );
-   }
-   else {
-      $self->{earlier_ids} = [];
-      $self->{seen_earlier_ids} = {};
-   }
-
    return $self;
 }
 
@@ -371,42 +360,41 @@ See L<VCP::Dest/backfill> as well.
 
 sub is_base_rev {
    my $self = shift ;
-   return ! defined $self->{action}
-      || $self->{action} eq "digest";
-}
 
-
-=item is_real_rev
-
-Returns true for base revisions and for "add" or "edit" revisions, all
-of which must be able to be checked out from the source repository.
-
-=cut
-
-sub is_real_rev {
-   my $self = shift;
-   return $self->is_base_rev
-      || $self->{action} eq "add"
-      || $self->{action} eq "edit";
+   return ! defined $self->{action};
 }
 
 
 =item is_placeholder_rev
 
-Returns TRUE if this is a branch or clone revision.  Placeholder revisions
-are used to record branch points without causing edits and to perform
-cloning operations.
+Returns TRUE if this is a placeholder revision.  Placeholder revisions
+are used to record branch points for files that have not been altered on
+their branches.
+
+This occurs when reading CVS repositories and finding files that have
+branch tags but no revisions on the branch.
+
+A placeholder revision has an action of "placeholder".
 
 Note that placeholders may have rev_id and change_id fields, but they
-are likely to be fabricated as they often don't actually exist in the
-source repository.  For instance, CVS does not record a revision number
-for a branch operation unless the file has been alterde on the branch.
+may be malformed; they are present for sorting purposes only and should
+be ignored by the destination repository.
+
+Placeholders may not be present for branches which have files on them.
 
 =cut
 
 sub is_placeholder_rev {
-   my $a = shift->{action};
-   return $a eq "branch" || $a eq "clone";
+   my $self = shift ;
+
+   my $a = $self->{action};
+
+   return defined $a
+      && (
+         $a eq "placeholder"
+         || $a eq "branch"
+         || $a eq "clone"
+      );
 }
 
 
@@ -429,7 +417,11 @@ branch before altering any files on it.
 =cut
 
 sub is_branch_rev {
-   return shift->{action} eq "branch";
+   my $self = shift ;
+
+   my $a = $self->{action};
+
+   return defined $a && $a eq "branch";
 }
 
 
@@ -444,7 +436,11 @@ this writing, VSS may by the time you read this.
 =cut
 
 sub is_clone_rev {
-   return shift->{action} eq "clone";
+   my $self = shift ;
+
+   my $a = $self->{action};
+
+   return defined $a && $a eq "clone";
 }
 
 
@@ -458,16 +454,15 @@ sub base_revify {
    my $self = shift ;
 
    $self->set_labels;
-   $self->{action} = "digest";
-   $self->{$_} = undef for qw(
+   $self->{$_} =undef for qw(
       p4_info
       cvs_info
       time
       mod_time
       user_id
       comment
+      action
       previous_id
-      from_id
    );
 }
 
@@ -553,7 +548,7 @@ sub get_source_file {
 }
 
 
-=item set_labels
+=item labels
 
    $r->set_labels( \@labels ) ;  ## pass an array ref for speed
    @labels = $r->labels ;
@@ -571,6 +566,7 @@ any moment.
 =item add_label
 
   $r->add_label( $label ) ;
+  $r->add_label( @labels ) ;
 
 Marks one or more labels as being associated with this revision of a file.
 
@@ -578,70 +574,26 @@ Marks one or more labels as being associated with this revision of a file.
 
 sub add_label {
    my $self = shift ;
-   push @{$self->{labels}}, $_[0]
-      unless $self->{seen_labels}->{$_[0]}++;
+   for ( @_ ) {
+      push @{$self->{labels}}, $_
+         unless $self->{seen_labels}->{$_}++;
+   }
    return ;
 }
 
 
 sub labels {
-   BUG "VCP::Rev::labels() is not a setter any more" if @_ > 1;
-   @{shift->{labels}};
+   goto &set_labels if @_ > 1;
+   my $self = shift;
+   @{$self->{labels}};
 }
 
 
 sub set_labels {
    my $self = shift;
+   @{$self->{labels}} = ();
    %{$self->{seen_labels}} = ();
-   @{$self->{labels}} = grep !$self->{seen_labels}->{$_}++,
-      ( @_ == 1 && ref $_[0] )
-         ? @{shift()}
-         : @_;
-}
-
-
-=item set_earlier_ids
-
-   $r->set_earlier_ids( \@earlier_ids );  ## pass an array ref for speed
-   $r->set_earlier_ids( @earlier_ids );   ## or now
-   @earlier_ids = $r->earlier_ids ;
-
-Sets/gets earlier_ids associated with a revision.  If a earlier_id is
-applied multiple times, it will only be returned once.  This feature
-means that the automatic earlier_id generation code for r_... revision
-and ch_... change earlier_ids won't add additional copies of earlier_ids
-that were already applied to this revision in the source repository.
-
-Returns earlier_ids in an unpredictible order, which happens to be sorted for
-now.  This sorting is purely for logging purposes and may disappear at
-any moment.
-
-=item add_earlier_id
-
-  $r->add_earlier_id( $earlier_id ) ;
-
-Marks one or more earlier_ids as being associated with this revision of a file.
-
-=cut
-
-sub add_earlier_id {
-   my $self = shift;
-   push @{$self->{earlier_ids}}, $_[0]
-      unless $self->{seen_earlier_ids}->{$_[0]}++;
-   return;
-}
-
-
-sub earlier_ids { @{shift->{earlier_ids}} }
-
-
-sub set_earlier_ids {
-   my $self = shift;
-   %{$self->{seen_earlier_ids}} = ();
-   @{$self->{earlier_ids}} = grep !$self->{seen_earlier_ids}->{$_}++,
-      ( @_ == 1 && ref $_[0] )
-         ? @{shift()}
-         : @_;
+   $self->add_label( map ref() ? @$_ : $_, @_ );
 }
 
 
@@ -658,9 +610,7 @@ an ISO8601 formatted date.  Exportable:
 
 sub iso8601format {
    die "time parameter missing" unless @_;
-   my $t = shift;
-   return undef unless $t;
-   my @f = reverse( (gmtime $t)[0..5] ) ;
+   my @f = reverse( (gmtime shift)[0..5] ) ;
    $f[0] += 1900 ;
    $f[1] ++ ; ## Month of year needs to be 1..12
    return sprintf( "%04d-%02d-%02d %02d:%02d:%02dZ", @f ) ;
@@ -700,17 +650,17 @@ sub as_string {
              ? sprintf "%-6s", $self->$_() # 6 == length "delete"
              : $self->$_(),
          (
-            qw( id action change_id branch_id type ),
+            qw( name rev_id change_id branch_id type ),
             $self->is_base_rev
                ? ()
-               : qw( time user_id comment ),
+               : qw( action time user_id comment ),
          )
       )
    ) ;
 
    return $self->is_base_rev
-      ? sprintf( qq{%s %s @%s <%s> (%s)}, @v )
-      : sprintf( qq{%s %s @%s <%s> (%s) %s %s "%s"}, @v );
+      ? sprintf( qq{%s#%s @%s <%s> (%s) BASE REV}, @v )
+      : sprintf( qq{%s#%s @%s <%s> (%s) %s %s %s "%s"}, @v );
 }
 
 =back
