@@ -539,7 +539,7 @@ sub get_source_file {
    ( $r ) = @_ ;
 
    BUG "can't check out ", $r->as_string, "\n"
-      unless $r->is_real_rev;
+      unless $r->is_base_rev || $r->action eq "add" || $r->action eq "edit";
 
    my $wp = $self->work_path( "revs", $r->source_name, $r->source_rev_id ) ;
    $self->mkpdir( $wp ) ;
@@ -579,7 +579,7 @@ sub _concoct_cloned_rev {
 
    my $pr = VCP::Rev->new(
       action               => "clone",
-      id                   => $r->id . "<$branch_tag>",
+      id                   => $r->id . "<$branch_tag clone>",
       name                 => $r->source_name,
       source_name          => $r->source_name,
       source_filebranch_id => $filebranch_id,
@@ -590,14 +590,7 @@ sub _concoct_cloned_rev {
       rev_id               => $r->source_rev_id,
       source_rev_id        => $r->source_rev_id,
       user_id              => $r->user_id,
-      $r->action eq "branch"
-         ? (
-            previous_id => $r->id
-         )
-         : (
-            previous_id => $r->previous_id . "<$branch_tag>",
-            from_id     => $r->id,
-         ),
+      previous_id          => $r->id,
       comment              => $r->is_branch_rev
          ? join( "",
             "[vcp] create branch '",
@@ -789,14 +782,14 @@ sub queue_parsed_revs {
             comment         => undef,
             previous_rev_id => $rev_id,
          };
-
          $placeholder_rev->{rev_id} =~ s{\.\d+\z}{.0};
+
          $self->compute_branch_metadata( $file_data, $placeholder_rev );
 
-         next unless $self->rev_mode( $placeholder_rev->{filebranch_id}, $branch_rev_id );
-         my $mode = $self->rev_mode( $placeholder_rev->{filebranch_id}, $placeholder_rev->{rev_id} );
+         my $mode = $self->rev_mode( $rev->{filebranch_id}, $rev_id );
          next unless $mode;
-         $placeholder_rev->{mode} = $mode;
+         $rev->{mode} = $mode;
+
          $placeholder_rev->{comment} =
             "[vcp] create branch '$placeholder_rev->{branch_id}'";
          $self->_create_rev( $file_data, $placeholder_rev );
@@ -1633,7 +1626,7 @@ sub parse_cvs_log_output {
          debug "$state [$foo]";
       }
 
-      if ( /^={77}$/ ) {
+      if ( /^={50,}$/ ) {
          debug "=======" if debugging && debug_parser;
          if ( $h ) {
             $h == $file_data
@@ -1643,7 +1636,7 @@ sub parse_cvs_log_output {
          $saw_equals = 1;
          $h = undef;
       }
-      elsif ( /^-{28}$/ ) {
+      elsif ( /^-{25,}$/ ) {
          debug "-----" if debugging && debug_parser;
          ## There's at least one CVS repository out there with
          ## munged revs that results in a "====" line followed by
@@ -1824,7 +1817,7 @@ sub _create_rev {
           && $file_data->{keyword} =~ /[o|b]/
       )
          ? "binary"
-         : "text";
+          : "text";
 
    my $rev_id = $rev_data->{rev_id};
 
