@@ -159,7 +159,7 @@ use VCP::Utils::p4;
 #use base qw( VCP::Source VCP::Utils::p4 ) ;
 #use fields (
 #   'P4_REPO_CLIENT',       ## Set by p4_parse_repo_spec in VCP::Utils::p4
-#   'P4_REP_DESC',          ## Results of the 'p4 info' command
+#   'P4_INFO',              ## Results of the 'p4 info' command
 #   'P4_RUN_P4D',           ## whether --run-p4d specified
 #   'P4_LABEL_CACHE',       ## ->{$name}->{$rev} is a list of labels for that rev
 #   'P4_MAX',               ## The last change number needed
@@ -251,12 +251,6 @@ sub init {
 
    $self->load_p4_info ;
    $self->load_p4_branches ;
-
-   warn "vcp: METADATA LOSS WARNING: p4 server version ",
-       $self->p4_server_version_number,
-       " does not export submit times (no filelog -t option)\n"
-       unless $self->has_filelog_t_option;
-
 }
 
 
@@ -272,34 +266,11 @@ sub ui_set_p4d_dir {
 }
 
 
-## Note: the next two routins are designed to be factored in to
-## VCP::Utils::p4 if & when the dest needs them.
 sub load_p4_info {
    my $self = shift ;
 
    my $errors = '' ;
-   $self->p4( ['info'], undef, \$self->{P4_REP_DESC} ) ;
-}
-
-
-sub p4_server_version_number {
-    my $self = shift;
-    die "No \"p4 info\" output to get server version from\n"
-        if empty $self->{P4_REP_DESC};
-    die "Can't parse server version from \"p4 info\" output:\n",
-        $self->{P4_REP_DESC}
-        unless $self->{P4_REP_DESC}
-            =~ m{^Server\s+version:.*\/([12]\d{3}\.\d+)/}m;
-    return $1;
-}
-
-
-sub has_filelog_t_option {
-    my $self = shift;
-    $self->{P4_HAS_FILELOG_T_OPTION} =
-        ($self->p4_server_version_number ge "2002.2")
-        unless defined $self->{P4_HAS_FILELOG_T_OPTION};
-    return $self->{P4_HAS_FILELOG_T_OPTION};
+   $self->p4( ['info'], undef, \$self->{P4_INFO} ) ;
 }
 
 
@@ -341,16 +312,16 @@ sub has_filelog_t_option {
 # This next regexp is used to parse the lines beginning "... #"
 
 my $filelog_rev_info_re = qr{
-   \G                       # Use with /gc!!
+   \G                  # Use with /gc!!
    ^\.\.\.\s+
-   \#(\d+)\s+               # Revision
-   change\s+(\d+)\s+        # Change nubmer
-   (\S+)\s+                 # Action
-   on\s+                    ### 'on '
-   ([\d/]+(?:\s[\d:]+)?)\s+ # date/date-time
-   by\s+                    ### 'by '
-   (\S(?:.*?\S))\s+         # user id.  Undelimited, so hope for best
-   \((\S+?)\)               # type
+   \#(\d+)\s+          # Revision
+   change\s+(\d+)\s+   # Change nubmer
+   (\S+)\s+            # Action
+   \S+\s+              ### 'on '
+   (\S+)\s+            # date
+   \S+\s+              ### 'by '
+   (\S(?:.*?\S))\s+    # user id.  Undelimited, so hope for best
+   \((\S+?)\)          # type
    .*\r?\n
 }mx ;
 
@@ -475,12 +446,6 @@ sub p4_filelog_parser {
             ? "binary"
             : "text";
 
-         $action = "edit"
-            if $action !~ /^(add|branch|delete)$/;
-            ## There are only add, branch, edit and delete actions
-            ## in VCP::Dest::* drivers at this time.
-
-
          $r = VCP::Rev->new(
             id                   => $id,
             action               => $action,
@@ -556,14 +521,11 @@ sub scan_metadata {
 
    $self->{P4_SPECS_TO_SCAN} = [ $spec ];
 
-   my @opts;
-   push @opts, "-t" if $self->has_filelog_t_option;
-
    while ( @{$self->{P4_SPECS_TO_SCAN}} ) {
       my $s = shift @{$self->{P4_SPECS_TO_SCAN}};
 
       $self->p4(
-         [ "filelog", "-m", $delta, @opts, "-l", $s ],
+         [ "filelog", "-m", $delta, "-l", $s ],
          undef,
          sub { $self->p4_filelog_parser( @_ ) },
          {
@@ -744,9 +706,7 @@ sub get_source_file {
 
    ( $r ) = @_ ;
    BUG "can't check out ", $r->as_string, "\n"
-      unless $r->is_real_rev;
-      ## Note that "integrate" is treated as an "edit" for this version
-      ## of VCP
+      unless $r->is_base_rev || $r->action eq "add" || $r->action eq "edit";
 
    my $fn  = $r->source_name ;
    my $rev = $r->source_rev_id ;
@@ -778,7 +738,7 @@ sub handle_header {
    my ( $header ) = @_ ;
 
    $header->{rep_type} = 'p4' ;
-   $header->{rep_desc} = $self->{P4_REP_DESC} ;
+   $header->{rep_desc} = $self->{P4_INFO} ;
    $header->{rev_root} = $self->rev_root ;
 
    my $tmp_db_loc = $self->tmp_dir;
@@ -835,17 +795,6 @@ specs are vs. how likely a problem this is to be.  We may also want to
 support "external" branch specs to allow deleted branch specs to be
 used.
 
-VCP::Source::p4 only emits "add", "branch", "delete" and "edit" actions;
-this is all most destinations can handle today.  Anything other than one
-of these four is converted to "edit".  Specifically, this means that
-when an integration into a file is found, this is treated as an edit.
-Transferring integration records that don't create branches is not
-implemented.
-
-p4 servers older than 2002.2 do not allow getting the submit date and
-time, only the submit *date*, so all changes will seem to happen at
-midnight.  Upgrate to the most recent p4d to solve this.
-
 =head1 SEE ALSO
 
 L<VCP::Dest::p4>, L<vcp>.
