@@ -280,14 +280,17 @@ sub get_source_file {
    ( $r ) = @_ ;
 
    die "can't check out ", $r->as_string, "\n"
-      unless $r->is_real_rev;
+      unless defined $r->is_base_rev || $r->action eq "add" || $r->action eq "edit";
 
-   return $self->_db_store_location(
-      'source_files',
-      $r->source_name,
-      $r->source_branch_id || "-",
-      $r->source_rev_id
-   );
+   return #VCP::RefCountedFile->new(
+      $self->_db_store_location(
+         'source_files',
+         $r->source_name,
+         $r->source_branch_id || "-",
+         $r->source_rev_id
+      )
+#   )
+   ;
 
       ## Pass ownership to the caller so it will be cleaned up when the
       ## caller is finished with it.
@@ -370,9 +373,6 @@ sub parse_revml_file {
                   if ( $tag eq 'label' ) {
                      push @{$stack[-1]->{labels}}, $elt->{TEXT} ;
                   }
-                  elsif ( $tag eq 'earlier_id' ) {
-                     push @{$stack[-1]->{earlier_ids}}, $elt->{TEXT} ;
-                  }
                   elsif ( $stack[-1]->{NAME} eq "revml" ) {
                      die "Header field $tag after first rev\n"
                         if $self->{HEADER_ENDED};
@@ -426,23 +426,31 @@ sub start_rev {
 ## extracted from the file.
 sub init_rev_meta {
    my $self = shift ;
-   my ( $depth ) = @_;
+   my ( $placeholder_type ) = @_;
 
-   my $rev_elt = $self->{STACK}->[-($depth||2)] ;
+   my $rev_elt = $self->{STACK}->[-2] ;
    my $r = VCP::Rev->new() ;
    ## All revml tag naes are lc, all internal data member names are uc
 #require Data::Dumper ; print Data::Dumper::Dumper( $self->{STACK} ) ;
 
    for my $key ( grep /^[a-z_0-9]+$/, keys %$rev_elt ) {
-      ## We know that all kids *in use today* of <rev> are pure PCDATA
-      ## Later, we'll need sub-attributes.
-      ## TODO: Flatten the element tree by preficing attribute names
-      ## with '@'?.
-      my $setter = "set_$key";
-      $r->$setter( $rev_elt->{$key} ) ;
+      if ( $key eq 'labels' ) {
+         $r->set_labels( $rev_elt->{labels} );
+      }
+      else {
+         ## We know that all kids *in use today* of <rev> are pure PCDATA
+	 ## Later, we'll need sub-attributes.
+	 ## TODO: Flatten the element tree by preficing attribute names
+	 ## with '@'?.
+         $r->$key( $rev_elt->{$key} ) ;
+      }
    }
+#require Data::Dumper ; print Data::Dumper::Dumper( $r ) ;
 
-   if ( !$r->is_placeholder_rev ) {
+   if ( defined $placeholder_type ) {
+      $r->action( $placeholder_type );
+   }
+   else {
       my $work_path =
          $self->_db_store_location(
             'source_files',
@@ -461,10 +469,30 @@ sub init_rev_meta {
 }
 
 
+sub start_delete {
+   my $self = shift ;
+
+   $self->init_rev_meta ;
+   $self->{REV}->set_action( "delete" ) ;
+   1;  ## prevent void context warning
+}
+
+
+sub start_move {
+   my $self = shift ;
+
+   $self->init_rev_meta ;
+   $self->{REV}->set_action( "move" ) ;
+   die "<move> unsupported" ;
+}
+
+
 sub start_content {
    my $self = shift ;
 
    $self->init_rev_meta ;
+#require Data::Dumper ; print Data::Dumper::Dumper( $self->{REV} ) ;
+   $self->{REV}->action( "edit" ) ;
    $self->{WORK_NAME} = $self->{FILES}->{$self->{REV}->id};
    $self->{UNDECODED_CONTENT} = "" ;
 
@@ -513,6 +541,7 @@ sub start_delta {
 
    $self->init_rev_meta ;
    my $r = $self->{REV} ;
+   $r->action( 'edit' ) ;
    $self->{WORK_NAME} = $self->_db_store_location(
       'source_files',
       $r->name,
@@ -624,10 +653,7 @@ sub end_digest {
 
    my $d = Digest::MD5->new() ;
    sysopen F, $work_path, O_RDONLY
-      or die "$! opening '$work_path' for digestion for ",
-         $r->as_string,
-         "\n";
-
+      or die "$! opening '$work_path' for digestion\n" ;
    ## See comment for binmode in start_content :)
    binmode F;
    $d->addfile( \*F ) ;
@@ -658,13 +684,17 @@ sub end_digest {
 }
 
 
+sub end_placeholder {
+   my $self = shift ;
+   $self->init_rev_meta( "placeholder" );
+}
+
+
 ## Having this and no sub rev_characters causes the parser to accumulate
 ## content.
 sub end_rev {
    my $self = shift ;
 
-   $self->init_rev_meta(1) unless $self->{REV};
-
    BUG "rev_root not set" unless defined $self->rev_root;
    my $abs_name = $self->rev_root . "/" . $self->{REV}->name;
    return if $abs_name !~ $self->{FILESPEC_RE};
