@@ -13,24 +13,13 @@ VCP::Utils::p4 - utilities for dealing with the p4 command
 A mix-in class providing methods shared by VCP::Source::p4 and VCP::Dest::p4,
 mostly wrappers for calling the p4 command.
 
-=head1 P4::Client Support (Experimental)
+If the P4::Client module is detected, this will be used to connect to
+the p4d instead of the command line client.  If this causes undesirable
+results, set the environment variable VCPP4API to "0" (zero):
 
-If the P4::Client module is installed, it can be used to connect to
-the p4d instead of the command line client.  While significantly faster,
-this mode is not ready for prime time due to unresolved issues with
-the underlying C++ API (it's been a while since I tested it, but
-as I recall, there was a reply buffer sequencing problem and a request
-buffer size limitation that was hard to know how to work around.
+   C:\> set VCPP4API=0  ## On Win32
 
-P4::Client's Perl-to-C++ adapter layer ("XS" layer) also needed some
-modification to support full access.
-
-To experiment with this mode, try doing the following before running
-VCP:
-
-   C:\> set VCPP4API=1  ## On Win32
-
-   $ export VCPP4API=1  ## Depends on your shell
+   $ export VCPP4API=0  ## Depends on your shell
 
 =cut
 
@@ -45,7 +34,11 @@ use VCP::Debug qw( :debug :profile ) ;
 use VCP::Logger qw( lg pr );
 use VCP::Utils qw( empty is_win32 shell_quote xchdir start_dir_rel2abs );
 
-use constant use_p4_api_if_present => $ENV{VCPP4API};
+use constant use_p4_api_if_present => (
+        empty( $ENV{VCPP4API} ) || $ENV{VCPP4API}
+    )
+        ? 1
+        : 0;
 
 use constant have_p4_api => use_p4_api_if_present
    && eval "require P4::Client; require P4::UI";
@@ -694,9 +687,9 @@ Returns a list of known clients.
 sub p4_clients {
    my $self = shift ;
 
+   my $clients ;
    pr "running p4 clients";
-   $self->p4( [ "clients", ], undef, \my $clients ) ;
-   return () if empty $clients;
+   $self->p4( [ "clients", ], undef, \$clients ) ;
    return map { /^Client (\S*)/ ; $1 } split /\n/m, $clients ;
 }
 
